@@ -1,6 +1,4 @@
-# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
-# pylint: skip-file
-# pylint: skip-file
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
 # Copyright 2013 jno <jno@pisem.net>
 # Copyright 2016 Dmitry Nezhevenko <dion@dion.org.ua>
@@ -27,25 +25,25 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-import StringIO
-from duplicity import log, globals
-from duplicity import progress
-import duplicity.backend
-from duplicity.errors import BackendException
-import os
-import sys
-import traceback
-import urllib
-import re
 
-from dropbox import Dropbox
-from dropbox.exceptions import AuthError, BadInputError, ApiError
-from dropbox.files import UploadSessionCursor, CommitInfo, WriteMode, \
-    GetMetadataError, DeleteError, UploadSessionLookupError, ListFolderError
-from dropbox.oauth import DropboxOAuth2FlowNoRedirect
-from requests.exceptions import ConnectionError
+import io
+import os
+import re
+import sys
 import time
-from duplicity.globals import num_retries
+import traceback
+import urllib.error
+import urllib.parse
+import urllib.request
+
+from duplicity import (
+    log,
+    config,
+)
+from duplicity import progress
+from duplicity.errors import BackendException
+from requests.exceptions import ConnectionError  # pylint: disable=redefined-builtin
+import duplicity.backend
 
 # This is chunk size for upload using Dpbx chumked API v2. It doesn't
 # make sense to make it much large since Dpbx SDK uses connection pool
@@ -61,31 +59,33 @@ DPBX_AUTORENAMED_FILE_RE = re.compile(r' \([0-9]+\)\.[^\.]+$')
 
 
 def log_exception(e):
-    log.Error('Exception [%s]:' % (e,))
-    f = StringIO.StringIO()
+    log.Error(f'Exception [{e}]:')
+    f = io.StringIO()
     traceback.print_exc(file=f)
     f.seek(0)
     for s in f.readlines():
-        log.Error('| ' + s.rstrip())
+        log.Error(f"| {s.rstrip()}")
     f.close()
 
 
-def command(login_required=True):
+def command(login_required=True):  # pylint: disable=unused-argument
     """a decorator for handling authentication and exceptions"""
+
     def decorate(f):
         def wrapper(self, *args):
             try:
                 return f(self, *args)
             except ApiError as e:
                 log_exception(e)
-                raise BackendException('dpbx api error "%s"' % (e,))
+                raise BackendException(f'dpbx api error "{e}"')
             except Exception as e:
                 log_exception(e)
-                log.Error('dpbx code error "%s"' % (e,), log.ErrorCode.backend_code_error)
+                log.Error(f'dpbx code error "{e}"', log.ErrorCode.backend_code_error)
                 raise
 
         wrapper.__doc__ = f.__doc__
         return wrapper
+
     return decorate
 
 
@@ -95,6 +95,35 @@ class DPBXBackend(duplicity.backend.Backend):
     def __init__(self, parsed_url):
         duplicity.backend.Backend.__init__(self, parsed_url)
 
+        global Dropbox
+        global AuthError, BadInputError, ApiError
+        global UploadSessionCursor, CommitInfo
+        global WriteMode, GetMetadataError
+        global DeleteError, UploadSessionLookupError
+        global ListFolderError
+        global DropboxOAuth2FlowNoRedirect
+        try:
+            from dropbox import Dropbox
+            from dropbox.exceptions import (
+                AuthError,
+                BadInputError,
+                ApiError,
+            )
+            from dropbox.files import (
+                UploadSessionCursor,
+                CommitInfo,
+                WriteMode,
+                GetMetadataError,
+                DeleteError,
+                UploadSessionLookupError,
+                ListFolderError,
+            )
+            from dropbox.oauth import DropboxOAuth2FlowNoRedirect
+        except ImportError as e:
+            raise BackendException(f"""This backend requires the dropbox package version 6.9.0
+To install use "sudo pip install dropbox==6.9.0"
+Exception: {str(e)}""")
+
         self.api_account = None
         self.api_client = None
         self.auth_flow = None
@@ -104,9 +133,9 @@ class DPBXBackend(duplicity.backend.Backend):
     def user_authenticated(self):
         try:
             account = self.api_client.users_get_current_account()
-            log.Debug("User authenticated as ,%s" % account)
+            log.Debug(f"User authenticated as ,{account}")
             return True
-        except:
+        except Exception as e:
             log.Debug('User not authenticated')
             return False
 
@@ -114,14 +143,13 @@ class DPBXBackend(duplicity.backend.Backend):
         return os.environ.get('DPBX_ACCESS_TOKEN', None)
 
     def save_access_token(self, access_token):
-        raise BackendException('dpbx: Please set DPBX_ACCESS_TOKEN=\"%s\" environment variable' %
-                               access_token)
+        raise BackendException(f'dpbx: Please set DPBX_ACCESS_TOKEN="{access_token}" environment variable')
 
     def obtain_access_token(self):
         log.Info("dpbx: trying to obtain access token")
         for env_var in ['DPBX_APP_KEY', 'DPBX_APP_SECRET']:
             if env_var not in os.environ:
-                raise BackendException('dpbx: %s environment variable not set' % env_var)
+                raise BackendException(f'dpbx: {env_var} environment variable not set')
 
         app_key = os.environ['DPBX_APP_KEY']
         app_secret = os.environ['DPBX_APP_SECRET']
@@ -133,18 +161,18 @@ class DPBXBackend(duplicity.backend.Backend):
         auth_flow = DropboxOAuth2FlowNoRedirect(app_key, app_secret)
         log.Debug('dpbx,auth_flow.start()')
         authorize_url = auth_flow.start()
-        print
-        print '-' * 72
-        print "1. Go to: " + authorize_url
-        print "2. Click \"Allow\" (you might have to log in first)."
-        print "3. Copy the authorization code."
-        print '-' * 72
-        auth_code = raw_input("Enter the authorization code here: ").strip()
+        print()
+        print('-' * 72)
+        print(f"1. Go to: {authorize_url}")
+        print("2. Click \"Allow\" (you might have to log in first).")
+        print("3. Copy the authorization code.")
+        print('-' * 72)
+        auth_code = input("Enter the authorization code here: ").strip()
         try:
-            log.Debug('dpbx,auth_flow.finish(%s)' % auth_code)
+            log.Debug(f'dpbx,auth_flow.finish({auth_code})')
             authresult = auth_flow.finish(auth_code)
         except Exception as e:
-            raise BackendException('dpbx: Unable to obtain access token: %s' % e)
+            raise BackendException(f'dpbx: Unable to obtain access token: {e}')
         log.Info("dpbx: Authentication successfull")
         self.save_access_token(authresult.access_token)
 
@@ -157,10 +185,10 @@ class DPBXBackend(duplicity.backend.Backend):
         try:
             log.Debug('dpbx,users_get_current_account([token])')
             self.api_account = self.api_client.users_get_current_account()
-            log.Debug("dpbx,%s" % self.api_account)
+            log.Debug(f"dpbx,{self.api_account}")
 
         except (BadInputError, AuthError) as e:
-            log.Debug('dpbx,exception: %s' % e)
+            log.Debug(f'dpbx,exception: {e}')
             log.Info("dpbx: Authentication failed. Trying to obtain new access token")
 
             self.obtain_access_token()
@@ -169,10 +197,9 @@ class DPBXBackend(duplicity.backend.Backend):
             # So this line should not be reached
             raise BackendException("dpbx: Please update DPBX_ACCESS_TOKEN and try again")
 
-        log.Info("dpbx: Successfully authenticated as %s" %
-                 self.api_account.name.display_name)
+        log.Info(f"dpbx: Successfully authenticated as {self.api_account.name.display_name}")
 
-    def _error_code(self, operation, e):
+    def _error_code(self, operation, e):  # pylint: disable=unused-argument
         if isinstance(e, ApiError):
             err = e.error
 
@@ -186,8 +213,8 @@ class DPBXBackend(duplicity.backend.Backend):
 
     @command()
     def _put(self, source_path, remote_filename):
-        remote_dir = urllib.unquote(self.parsed_url.path.lstrip('/'))
-        remote_path = '/' + os.path.join(remote_dir, remote_filename).rstrip()
+        remote_dir = urllib.parse.unquote(self.parsed_url.path.lstrip('/'))
+        remote_path = f"/{os.path.join(remote_dir, remote_filename.decode()).rstrip()}"
 
         file_size = os.path.getsize(source_path.name)
         progress.report_transfer(0, file_size)
@@ -200,11 +227,9 @@ class DPBXBackend(duplicity.backend.Backend):
 
         # A few sanity checks
         if res_metadata.path_display != remote_path:
-            raise BackendException('dpbx: result path mismatch: %s (expected: %s)' %
-                                   (res_metadata.path_display, remote_path))
+            raise BackendException(f'dpbx: result path mismatch: {res_metadata.path_display} (expected: {remote_path})')
         if res_metadata.size != file_size:
-            raise BackendException('dpbx: result size mismatch: %s (expected: %s)' %
-                                   (res_metadata.size, file_size))
+            raise BackendException(f'dpbx: result size mismatch: {res_metadata.size} (expected: {file_size})')
 
     def put_file_small(self, source_path, remote_path):
         if not self.user_authenticated():
@@ -213,14 +238,14 @@ class DPBXBackend(duplicity.backend.Backend):
         file_size = os.path.getsize(source_path.name)
         f = source_path.open('rb')
         try:
-            log.Debug('dpbx,files_upload(%s, [%d bytes])' % (remote_path, file_size))
+            log.Debug(f'dpbx,files_upload({remote_path}, [{int(file_size)} bytes])')
 
             res_metadata = self.api_client.files_upload(f.read(), remote_path,
                                                         mode=WriteMode.overwrite,
                                                         autorename=False,
                                                         client_modified=None,
                                                         mute=True)
-            log.Debug('dpbx,files_upload(): %s' % res_metadata)
+            log.Debug(f'dpbx,files_upload(): {res_metadata}')
             progress.report_transfer(file_size, file_size)
             return res_metadata
         finally:
@@ -234,10 +259,9 @@ class DPBXBackend(duplicity.backend.Backend):
         f = source_path.open('rb')
         try:
             buf = f.read(DPBX_UPLOAD_CHUNK_SIZE)
-            log.Debug('dpbx,files_upload_session_start([%d bytes]), total: %d' %
-                      (len(buf), file_size))
+            log.Debug(f'dpbx,files_upload_session_start([{len(buf)} bytes]), total: {int(file_size)}')
             upload_sid = self.api_client.files_upload_session_start(buf)
-            log.Debug('dpbx,files_upload_session_start(): %s' % upload_sid)
+            log.Debug(f'dpbx,files_upload_session_start(): {upload_sid}')
             upload_cursor = UploadSessionCursor(upload_sid.session_id, f.tell())
             commit_info = CommitInfo(remote_path, mode=WriteMode.overwrite,
                                      autorename=False, client_modified=None,
@@ -247,7 +271,7 @@ class DPBXBackend(duplicity.backend.Backend):
 
             requested_offset = None
             current_chunk_size = DPBX_UPLOAD_CHUNK_SIZE
-            retry_number = globals.num_retries
+            retry_number = config.num_retries
             is_eof = False
 
             # We're doing our own error handling and retrying logic because
@@ -269,25 +293,24 @@ class DPBXBackend(duplicity.backend.Backend):
                     # reset temporary status variables
                     requested_offset = None
                     current_chunk_size = DPBX_UPLOAD_CHUNK_SIZE
-                    retry_number = globals.num_retries
+                    retry_number = config.num_retries
 
                     if not is_eof:
                         assert len(buf) != 0
-                        log.Debug('dpbx,files_upload_sesssion_append([%d bytes], offset=%d)' %
-                                  (len(buf), upload_cursor.offset))
+                        log.Debug(f'dpbx,files_upload_sesssion_append([{len(buf)} bytes], '
+                                  f'offset={int(upload_cursor.offset)})')
                         self.api_client.files_upload_session_append(buf,
                                                                     upload_cursor.session_id,
                                                                     upload_cursor.offset)
                     else:
-                        log.Debug('dpbx,files_upload_sesssion_finish([%d bytes], offset=%d)' %
-                                  (len(buf), upload_cursor.offset))
+                        log.Debug(f'dpbx,files_upload_sesssion_finish([{len(buf)} bytes], '
+                                  f'offset={int(upload_cursor.offset)})')
                         res_metadata = self.api_client.files_upload_session_finish(buf,
                                                                                    upload_cursor,
                                                                                    commit_info)
 
                     upload_cursor.offset = f.tell()
-                    log.Debug('progress: %d of %d' % (upload_cursor.offset,
-                                                      file_size))
+                    log.Debug(f'progress: {int(upload_cursor.offset)} of {int(file_size)}')
                     progress.report_transfer(upload_cursor.offset, file_size)
                 except ApiError as e:
                     error = e.error
@@ -298,8 +321,8 @@ class DPBXBackend(duplicity.backend.Backend):
                         # expected offset from server and it's enough to just
                         # seek() and retry again
                         new_offset = error.get_incorrect_offset().correct_offset
-                        log.Debug('dpbx,files_upload_session_append: incorrect offset: %d (expected: %s)' %
-                                  (upload_cursor.offset, new_offset))
+                        log.Debug(f'dpbx,files_upload_session_append: incorrect offset: {int(upload_cursor.offset)} '
+                                  f'(expected: {new_offset})')
                         if requested_offset is not None:
                             # chunk failed even after seek attempt. Something
                             # strange and no safe way to recover
@@ -310,7 +333,7 @@ class DPBXBackend(duplicity.backend.Backend):
                         continue
                     raise
                 except ConnectionError as e:
-                    log.Debug('dpbx,files_upload_session_append: %s' % e)
+                    log.Debug(f'dpbx,files_upload_session_append: {e}')
 
                     retry_number -= 1
 
@@ -332,7 +355,7 @@ class DPBXBackend(duplicity.backend.Backend):
             if f.tell() != file_size:
                 raise BackendException('dpbx: something wrong')
 
-            log.Debug('dpbx,files_upload_sesssion_finish(): %s' % res_metadata)
+            log.Debug(f'dpbx,files_upload_sesssion_finish(): {res_metadata}')
             progress.report_transfer(f.tell(), file_size)
 
             return res_metadata
@@ -345,13 +368,12 @@ class DPBXBackend(duplicity.backend.Backend):
         if not self.user_authenticated():
             self.login()
 
-        remote_dir = urllib.unquote(self.parsed_url.path.lstrip('/'))
-        remote_path = '/' + os.path.join(remote_dir, remote_filename).rstrip()
+        remote_dir = urllib.parse.unquote(self.parsed_url.path.lstrip('/'))
+        remote_path = f"/{os.path.join(remote_dir, remote_filename.decode()).rstrip()}"
 
-        log.Debug('dpbx,files_download(%s)' % remote_path)
+        log.Debug(f'dpbx,files_download({remote_path})')
         res_metadata, http_fd = self.api_client.files_download(remote_path)
-        log.Debug('dpbx,files_download(%s): %s, %s' % (remote_path, res_metadata,
-                                                       http_fd))
+        log.Debug(f'dpbx,files_download({remote_path}): {res_metadata}, {http_fd}')
         file_size = res_metadata.size
         to_fd = None
         progress.report_transfer(0, file_size)
@@ -370,8 +392,7 @@ class DPBXBackend(duplicity.backend.Backend):
         # again. Since this check is free, it's better to have it here
         local_size = os.path.getsize(local_path.name)
         if local_size != file_size:
-            raise BackendException("dpbx: wrong file size: %d (expected: %d)" %
-                                   (local_size, file_size))
+            raise BackendException(f"dpbx: wrong file size: {int(local_size)} (expected: {int(file_size)})")
 
         local_path.setdata()
 
@@ -380,13 +401,13 @@ class DPBXBackend(duplicity.backend.Backend):
         # Do a long listing to avoid connection reset
         if not self.user_authenticated():
             self.login()
-        remote_dir = '/' + urllib.unquote(self.parsed_url.path.lstrip('/')).rstrip()
+        remote_dir = f"/{urllib.parse.unquote(self.parsed_url.path.lstrip('/')).rstrip()}"
 
-        log.Debug('dpbx.files_list_folder(%s)' % remote_dir)
+        log.Debug(f'dpbx.files_list_folder({remote_dir})')
         res = []
         try:
             resp = self.api_client.files_list_folder(remote_dir)
-            log.Debug('dpbx.list(%s): %s' % (remote_dir, resp))
+            log.Debug(f'dpbx.list({remote_dir}): {resp}')
 
             while True:
                 res.extend([entry.name for entry in resp.entries])
@@ -396,7 +417,7 @@ class DPBXBackend(duplicity.backend.Backend):
         except ApiError as e:
             if (isinstance(e.error, ListFolderError) and e.error.is_path() and
                     e.error.get_path().is_not_found()):
-                log.Debug('dpbx.list(%s): ignore missing folder (%s)' % (remote_dir, e))
+                log.Debug(f'dpbx.list({remote_dir}): ignore missing folder ({e})')
             else:
                 raise
 
@@ -410,10 +431,10 @@ class DPBXBackend(duplicity.backend.Backend):
         if not self.user_authenticated():
             self.login()
 
-        remote_dir = urllib.unquote(self.parsed_url.path.lstrip('/'))
-        remote_path = '/' + os.path.join(remote_dir, filename).rstrip()
+        remote_dir = urllib.parse.unquote(self.parsed_url.path.lstrip('/'))
+        remote_path = f"/{os.path.join(remote_dir, filename.decode()).rstrip()}"
 
-        log.Debug('dpbx.files_delete(%s)' % remote_path)
+        log.Debug(f'dpbx.files_delete({remote_path})')
         self.api_client.files_delete(remote_path)
 
         # files_permanently_delete seems to be better for backup purpose
@@ -429,12 +450,12 @@ class DPBXBackend(duplicity.backend.Backend):
     def _query(self, filename):
         if not self.user_authenticated():
             self.login()
-        remote_dir = urllib.unquote(self.parsed_url.path.lstrip('/'))
-        remote_path = '/' + os.path.join(remote_dir, filename).rstrip()
+        remote_dir = urllib.parse.unquote(self.parsed_url.path.lstrip('/'))
+        remote_path = f"/{os.path.join(remote_dir, filename.decode()).rstrip()}"
 
-        log.Debug('dpbx.files_get_metadata(%s)' % remote_path)
+        log.Debug(f'dpbx.files_get_metadata({remote_path})')
         info = self.api_client.files_get_metadata(remote_path)
-        log.Debug('dpbx.files_get_metadata(%s): %s' % (remote_path, info))
+        log.Debug(f'dpbx.files_get_metadata({remote_path}): {info}')
         return {'size': info.size}
 
     def check_renamed_files(self, file_list):
@@ -455,7 +476,7 @@ class DPBXBackend(duplicity.backend.Backend):
         log.Warn('')
         log.Warn('Affected files:')
         for x in bad_list:
-            log.Warn('\t%s' % x)
+            log.Warn(f'\t{x}')
         log.Warn('')
         log.Warn('In any case it\'s better to create full backup.')
         log.Warn('-' * 72)
